#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import datetime
import json

from manukaclient.v1 import external_ids

from manukaclient.tests.unit import utils
from manukaclient.tests.unit.v1 import fakes


class ExternalIdsTest(utils.TestCase):

    def setUp(self):
        super(ExternalIdsTest, self).setUp()
        self.cs = fakes.FakeClient()

    def test_update(self):
        new_user_id = '234'
        e = self.cs.external_ids.update(123, user_id=new_user_id)
        self.cs.assert_called('PATCH', '/v1/external-ids/123/',
                              json.dumps({'user_id': new_user_id}))
        self.assertIsInstance(e, external_ids.ExternalId)
        self.assertIsInstance(e.last_login, datetime.datetime)

    def test_delete(self):
        self.cs.external_ids.delete(123)
        self.cs.assert_called('DELETE', '/v1/external-ids/123/')



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from manukaclient import client
from manukaclient import exceptions
from manukaclient.v1 import external_ids
from manukaclient.v1 import users


class Client(object):
    """Client for the Nectar Allocations v1 API
    :param string session: session
    :type session: :py:class:`keystoneauth.adapter.Adapter`
    """

    def __init__(self, session=None, service_type='account', **kwargs):
        """Initialize a new client for the Manuka v1 API."""
        if session is None:
            raise exceptions.ClientException(
                message='Session is required argument')
        self.http_client = client.SessionClient(
            session, service_type=service_type, **kwargs)
        self.external_ids = external_ids.ExternalIdManager(self.http_client)
        self.pending_users = users.PendingUserManager(self.http_client)
        self.users = users.UserManager(self.http_client)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import logging

from osc_lib.command import command

from manukaclient import exceptions


class ExternalIdCommand(command.ShowOne):

    def get_parser(self, prog_name):
        parser = super(ExternalIdCommand, self).get_parser(prog_name)
        parser.add_argument(
            'id',
            metavar='<id>',
            help=('ID of external_id')
        )
        return parser


class ShowExternalId(ExternalIdCommand):
    """Show external id details."""

    log = logging.getLogger(__name__ + '.ShowExternalId')

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)', parsed_args)
        client = self.app.client_manager.account
        try:
            external_id = client.external_ids.get(parsed_args.id)
        except exceptions.NotFound as ex:
            raise exceptions.CommandError(str(ex))

        return self.dict2columns(external_id.to_dict())


class DeleteExternalId(ExternalIdCommand):
    """Delete external id."""

    log = logging.getLogger(__name__ + '.DeleteExternalId')

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)', parsed_args)
        client = self.app.client_manager.account
        try:
            client.external_ids.delete(parsed_args.id)
        except exceptions.NotFound as ex:
            raise exceptions.CommandError(str(ex))

        return [], []


class UpdateExternalId(ExternalIdCommand):
    """Update an external id."""

    log = logging.getLogger(__name__ + '.UpdateExternalId')

    def get_parser(self, prog_name):
        parser = super(UpdateExternalId, self).get_parser(prog_name)
        parser.add_argument(
            '--user-id',
            metavar='<user_id>',
            help=('User ID of external ID'))
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)', parsed_args)
        client = self.app.client_manager.account
        try:
            client.external_ids.update(
                parsed_args.id, user_id=parsed_args.user_id)
        except exceptions.NotFound as ex:
            raise exceptions.CommandError(str(ex))

        # Return the user as more useful
        user = client.users.get(parsed_args.user_id)

        return self.dict2columns(user.to_dict())



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import json

from manukaclient import base
from manukaclient.v1 import external_ids


class User(base.Resource):

    date_fields = ['registered_at', 'terms_accepted_at', 'last_login']

    def __init__(self, manager, info, loaded=False, resp=None):
        super(User, self).__init__(manager, info, loaded, resp)
        raw_external_ids = getattr(self, 'external_ids', [])
        self.external_ids = []
        for eid in raw_external_ids:
            self.external_ids.append(external_ids.ExternalId(manager, eid))

    def __repr__(self):
        return "<User %s>" % self.id


class UserManager(base.BasicManager):

    base_url = 'v1/users'
    resource_class = User

    def update(self, user_id, **kwargs):
        data = json.dumps(kwargs)
        return self._update('/%s/%s/' % (self.base_url, user_id), data=data,
                            headers={"content-type": "application/json"})

    def get_by_os(self, user_id):
        return self._get('/v1/users-os/%s/' % user_id)

    def search(self, query):
        return self._search('/%s/search/' % self.base_url,
                            data={'search': query})


class PendingUserManager(UserManager):

    base_url = 'v1/pending-users'

    def delete(self, user_id):
        return self._delete('/%s/%s/' % (self.base_url, user_id))



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import datetime
import json

from manukaclient.v1 import users

from manukaclient.tests.unit import utils
from manukaclient.tests.unit.v1 import fakes


class UsersTest(utils.TestCase):

    def setUp(self):
        super(UsersTest, self).setUp()
        self.cs = fakes.FakeClient()

    def test_user_list(self):
        ul = self.cs.users.list()
        self.cs.assert_called('GET', '/v1/users/')
        for u in ul:
            self.assertIsInstance(u, users.User)
        self.assertEqual(3, len(ul))

    def test_user_get(self):
        u = self.cs.users.get(123)
        self.cs.assert_called('GET', '/v1/users/123/')
        self.assertIsInstance(u, users.User)
        self.assertEqual(123, u.id)
        self.assertIsInstance(u.registered_at, datetime.datetime)
        self.assertIsInstance(u.terms_accepted_at, datetime.datetime)

    def test_user_get_by_os(self):
        u = self.cs.users.get_by_os(123)
        self.cs.assert_called('GET', '/v1/users-os/123/')
        self.assertIsInstance(u, users.User)
        self.assertEqual(123, u.id)

    def test_update(self):
        new_orcid = 'new-orcid'
        u = self.cs.users.update(123, orcid=new_orcid)
        self.cs.assert_called('PATCH', '/v1/users/123/',
                              json.dumps({'orcid': new_orcid}))
        self.assertIsInstance(u, users.User)
        self.assertEqual(new_orcid, u.orcid)

    def test_search(self):
        query = 'needle'
        ul = self.cs.users.search(query)
        self.cs.assert_called('POST', '/v1/users/search/', {'search': query})
        for u in ul:
            self.assertIsInstance(u, users.User)


class PendingUsersTest(utils.TestCase):

    def setUp(self):
        super(PendingUsersTest, self).setUp()
        self.cs = fakes.FakeClient()

    def test_user_list(self):
        ul = self.cs.pending_users.list()
        self.cs.assert_called('GET', '/v1/pending-users/')
        for u in ul:
            self.assertIsInstance(u, users.User)
        self.assertEqual(3, len(ul))

    def test_user_get(self):
        u = self.cs.pending_users.get(123)
        self.cs.assert_called('GET', '/v1/pending-users/123/')
        self.assertIsInstance(u, users.User)
        self.assertEqual(123, u.id)

    def test_user_delete(self):
        self.cs.pending_users.delete(123)
        self.cs.assert_called('DELETE', '/v1/pending-users/123/')
